clear 
close all
%%
cd D:\Kopriva\MATLAB\work\HrZZ_2013_2015\Nonlinear_BSS\Scientific_Reports_2015
%%
%  SREP_11576
% 
% (c) Ivica Kopriva, April 2013 - June 2015. 
%
% This code reproduces results associated with Figures 1 to 4 and presented
% in the paper: I. Kopriva, M. Popovi Hadija, M. Hadija, G. Aralica,
% "Unsupervised segmentation of low-contrast multichannel images: discrimination
% of tissue components in microscopic images of unstained specimen",
% Scientific Reports 5, 11576; doi:10.1038/srep11576 (2015).
% 
% This work has been funded by the Croatian Science Foundation Grant
% 9.01/232 "Nonlinear component analysis with applications in 
%           chemometrics and pathology"
%
%%
clear
close all

%% Parameters setup
                       
% type of source signals
image_type = 6;        % 1 - synthetic image in Figure 1 
                       % 2 - RGB image of human liver with hepatocellular 
                       %     carcinoma in Figure 2a 
                       % 3 - RGB image of human liver with metastasis 
                       %     from colon cancer in Figure 3a 
                       % 4 - RGB image of human liver with metastasis 
                       %     from gastric cancer in Figure 4a 
                       % 5 - RGB image of mouse fatty liver in Figure S5a
                       % 6 - RGB image of mouse fatty liver in Figure S6a

%%  read or synthesise the image to be segmented
if image_type == 1
    S(:,:,1)=zeros(256,256);
    S(20:75,20:75,1)=1;
    S(:,:,2)=zeros(256,256);
    S(150:205,20:75,2)=1;
    S(:,:,3)=zeros(256,256);
    S(20:75,150:205,3)=1;
    S(:,:,4)=zeros(256,256);
    S(150:205,150:205,4)=1;
    S(:,:,5) = ones(256,256);
    S(:,:,5) = S(:,:,5).*(abs(1-S(:,:,1)));
    S(:,:,5) = S(:,:,5).*(abs(1-S(:,:,2)));
    S(:,:,5) = S(:,:,5).*(abs(1-S(:,:,3)));
    S(:,:,5) = S(:,:,5).*(abs(1-S(:,:,4)));  
    tmp=S;
    clear S
    [I1 I2 I3] = size(tmp);
    for i3=1:I3
        S(i3,:)=reshape(tmp(:,:,i3),1,I1*I2);
    end
    clear tmp
    S = max(0,S);
    a1 = [1.0 2.5574 2.7183]';
    a2 = [1.0 2.0 1.7854]';
    a3 = [0.7854 2.0 2.0]';
    a4 = [1.0 1.7854 2.0]';
    a5 = [1.0 2.7183 2.5574]';
    A = [a1 a2 a3 a4 a5];
    X=A*S;  % vectorized synthetic low-contrast RGB image     
    
    SNR = 70;           % add extremely samll amount of AWGN
  
    for n=1:3
        xv=var(X(n,:));
        nv = xv*power(10,-SNR/10);
        X(n,:) = max(0,X(n,:)+sqrt(nv)*randn(1,256*256));
    end
    X = max(0,X);
    
    xmax = max(max(X));
    X = X/xmax;
    
    D = 20;         % dimension of EKM-induced RKHS
    sig2 = 0.001;    % variance of Gaussian kernel    
    M = 5;                 % number of sources
    
    % color coded ground truth image
    b1 = [0.8 0.1 0.1]';
    b2 = [0.2 0.6 0.2]';
    b3 = [0.1 0.1 0.8]';
    b4 = [0.6 0.3 0.4]';
    b5 = [0.6 0.3 0.1]';
    B = [b1 b2 b3 b4 b5];
    Gt = B*S;  
elseif image_type == 2
    cd .\Figure_2
    G = imread('Figure_2a','jpg'); % RGB image of unstained specimen
    cd ..
    G = imresize(G,0.5);   % downsample to reduced computation complexity 
    [I1 I2 I3] = size(G);
    for i3=1:I3              % vectorized image of unstained specimen
        X(i3,:)=double(reshape(G(:,:,i3),1,I1*I2));
    end
    xmax = max(max(X));
    X = X/xmax;

    D = 50;        % dimension of EKM-induced RKHS
    sig2 = 0.1;    % variance of Gaussian kernel   
    M = 6;         % number of sources
    Fig_text='HCC: ';
elseif image_type == 3
    cd .\Figure_3
    G = imread('Figure_3a','jpg'); % RGB image of unstained specimen
    cd ..
    G = imresize(G,0.5);   % downsample to reduced computation complexity 
    [I1 I2 I3] = size(G);
    for i3=1:I3              % vectorized image of unstained specimen
        X(i3,:)=double(reshape(G(:,:,i3),1,I1*I2));
    end
    xmax = max(max(X));
    X = X/xmax;
    
    D = 50;             % dimension of EKM-induced RKHS
    sig2 = 0.1;         % variance of Gaussian kernel
    M = 6;              % number of sources
    Fig_text='Liver with colon cancer: ';
elseif image_type == 4
    cd .\Figure_4
    G = imread('Figure_4a','jpg'); % RGB image of unstained specimen
    cd ..
    G = imresize(G,0.5);   % downsample to reduced computation complexity 
    [I1 I2 I3] = size(G);
    for i3=1:I3              % vectorized image of unstained specimen
        X(i3,:)=double(reshape(G(:,:,i3),1,I1*I2));
    end
    xmax = max(max(X));
    X = X/xmax;
    
    D = 50;             % dimension of EKM-induced RKHS
    sig2 = 0.1;         % variance of Gaussian kernel
    M = 6;              % number of sources
    Fig_text='Liver with gastric cancer: ';
elseif image_type == 5
    cd .\Figure_S5
    G = imread('Figure_S5a','jpg'); % RGB image of unstained specimen
    cd ..
    G = imresize(G,0.5);   % downsample to reduced computation complexity 
    [I1 I2 I3] = size(G);
    for i3=1:I3              % vectorized image of unstained specimen
        X(i3,:)=double(reshape(G(:,:,i3),1,I1*I2));
    end
    xmax = max(max(X));
    X = X/xmax;
    
    D = 50;             % dimension of EKM-induced RKHS
    sig2 = 0.1;         % variance of Gaussian kernel
    M = 6;              % number of sources
    Fig_text='Mouse fatty liver: ';
elseif image_type == 6
    cd .\Figure_S6
    G = imread('Figure_S6a','jpg'); % RGB image of unstained specimen
    cd ..
    G = imresize(G,0.5);   % downsample to reduced computation complexity 
    [I1 I2 I3] = size(G);
    for i3=1:I3              % vectorized image of unstained specimen
        X(i3,:)=double(reshape(G(:,:,i3),1,I1*I2));
    end
    xmax = max(max(X));
    X = X/xmax;
    
    D = 50;             % dimension of EKM-induced RKHS
    sig2 = 0.1;         % variance of Gaussian kernel
    M = 6;              % number of sources
    Fig_text='Mouse fatty liver: ';    
end

%% nonlinear blind source separation
 tstart=tic;

 % eliminate small components
 Xn=sqrt(sum(X.*X));
 [xmax indmax] = max(Xn);
 Xn = Xn./xmax;
 P=Xn-1/100000; P=sign(P); P=(-P+1)/2;
 Xnp=Xn.*P;
 [xd,Ind]=setdiff(Xn,Xnp);
 XR = X(:,Ind);

 % find a basis V in the input pattern space by k-means clustering
 cnum=1e3;
 if (D <= 20)
     epscnum=60.0;
 else
     epscnum=150.0;  % enforce the basis with "small" condition number
 end
    
 while cnum > epscnum
      [SH,Vt]=kmeans(X',D,'distance','sqEuclidean','Replicates',1,'EmptyAction','singleton');
      V = Vt';
      cnum=cond(V)
 end
clear SH

% use Gaussian kernel based EKM to project input image on a subspace in RKHS 
           
g = waitbar(0,'Subspace projection ...');
for d=1:D
    P = X-repmat(V(:,d),1,I1*I2);   
    P = sum(P.*P);
    PSIX(d,:)=exp(-P/2/sig2);
    waitbar(d/D,g)
end
close(g)

if image_type == 1  % use the NMU algorithm for sparseness constrained NMF
    %
    % The MTALAB code is available for free download at:
    % https://sites.google.com/site/nicolasgillis/code
    % 
    % In particular, recursive NMU is available under section:
    % Nonnegtive matrix underapproximation (NMU) - b) recursive NMU
    %
    
    cd .\NMU_NMF_Nicolas_Gillis
    norm_type = 2;
    [St,A] = recursive_NMU(PSIX',M,norm_type,40);
    SH = St';
    cd ..
    % binarize decomposition resul
    SB = zeros(M,size(SH,2));
    for t=1:size(SH,2)
        [vm im]=max(SH(:,t));
        SB(im,t)=1.0;
    end
else                % use the NMF_L0 algorithm for sparseness constrained NMF    
    %
    % The MTALAB code is available for free download at:
    % https://www.spsc.tugraz.at/tools/nmf-l0-sparseness-constraints
    % 
        
   cd .\NMF_L0_Robert_Pehartz 
   % set NMFL0_H parameters
   options.K = M;           % number of components
   options.L = 1;           % number of overlapping sources
   options.numIter = 7;     % number of iterations

   %%% select sparse coder
   options.sparseCoder = @rsNNLS___;
   %%% select dictionary update method
   options.updateType = 'ANLS_FC';

   %%% number of dictionary updates
   options.numUpdateIter = 10;
   [AH,SH,INFO] = NMFL0_H(PSIX,options);
   cd ..
    % binarize decomposition resul
    SB = zeros(M,size(SH,2));
    for t=1:size(SH,2)
        [vm im]=max(SH(:,t));
        SB(im,t)=1.0;
    end
end
         
time_EKM_NMF_L0=toc(tstart)

%% display/store results
if image_type == 1
   for i=1:3
       tmp(:,:,i)=reshape(Gt(i,:),256,256);
   end
   
   figure
   imshow(tmp)
   title('Color coded ground truth image')
   
   %  resolve permutation ambuguity
   for m=1:M
       for n=1:M
           crc(n)=S(m,:)*SB(n,:)'/norm(S(m,:))/norm(SB(n,:));
       end
       [cm im]=max(crc);
       ind(m)=im;
   end
   SB = SB(ind,:);
   
   figure
   X=B*SB;  % back project segmented image in RGB color space
   for i=1:3
       tmp(:,:,i)=reshape(X(i,:),256,256);
   end
   imshow(tmp)
   title('Color coded segmented image')
   
   for m=1:M
       figure
       imshow(reshape(SB(m,:),256,256))
       title(strcat('SH',num2str(m)));
   end
else
    for i=1:size(SB,1)
        figure
        SBi=reshape(SB(i,:),I1,I2);
        imshow(SBi)
        title(strcat(Fig_text,'component ',num2str(i)));
        %  colorbar('vert')
        % set(gca,'Ytick',[]);
        % set(gca,'Xtick',[]);
        colormap('gray')
        %  imwrite(SBi, strcat('SH',num2str(i),'fIGURE_EKM_Gauss_50_NMF_L0_sig2_1em1.tiff'), 'tiff', 'Compression', 'none');
    end
end



